@@ -2,5 +2,6 @@ from .step import step
 from .review import review
 from .workflow import workflow
 from .models import ReviewStatus
+from .llm import prompt
 
-__all__ = [step, review, workflow, ReviewStatus]
+__all__ = [step, review, workflow, ReviewStatus, prompt]
