@@ -2,6 +2,7 @@ import os
 from typing import List
 
 import anthropic
+import instructor
 from anthropic.types import (
     TextBlockParam,
     ImageBlockParam,
@@ -36,7 +37,7 @@ class Agent:
 
     def __init__(self):
         """Create a new Agent instance."""
-        self.model = "claude-3-7-sonnet-20250219"
+        self.model = "claude-sonnet-4-20250514"
         self.max_iterations = 100  # Maximum number of actions to prevent infinite loops
 
         # Use API key from environment variable if not provided
@@ -49,3 +50,5 @@ class Agent:
             )
 
         self.client = anthropic.Anthropic(api_key=anthropic_api_key)
+        # provides structured output
+        self.instructor_client = instructor.from_anthropic(self.client)
