@@ -9,6 +9,8 @@ def normalize_locator_string(js_locator: str) -> str:
     - getByRole('textbox', { name: 'Customer name:' }) -> get_by_role('textbox', name="Customer name:")
     - getByText('Submit') -> get_by_text('Submit')
     - getByLabel('Email') -> get_by_label('Email')
+    - locator('x-details', { hasText: 'Details' }) -> locator('x-details', has_text="Details")
+    - getByRole('button', { includeHidden: true }) -> get_by_role('button', include_hidden=True)
     """
     if not js_locator:
         return js_locator
@@ -69,6 +71,10 @@ def normalize_locator_string(js_locator: str) -> str:
                 elif key.startswith("'") and key.endswith("'"):
                     key = key[1:-1]
 
+                # Convert camelCase keys to snake_case
+                # Handle common Playwright options
+                key = re.sub(r"([a-z])([A-Z])", r"\1_\2", key).lower()
+
                 # Convert single quotes to double quotes for string values
                 if value.startswith("'") and value.endswith("'"):
                     value = f'"{value[1:-1]}"'
