@@ -8,6 +8,7 @@ from playwright._impl._helper import locals_to_params
 from playwright._impl._locator import Locator as LocatorImpl
 from playwright.sync_api import Locator
 
+from va.constants import VA_DISABLE_FALLBACK
 from va.playwright.web_agent import WebAgent
 
 
@@ -54,40 +55,85 @@ class PromptBasedLocator:
         """Get the prompt-based locator, raising an exception if it can't be found."""
         # Create a WebAgent to find the element and get its locator
         agent = WebAgent(self.page)
-        command = f"Find the element for '{self.prompt}'. Use get_page_snapshot(), find the matching element, then use find_element_by_ref() to get its locator. You don't need to verify the locator by interaction."
+        command = f"Find the element for '{self.prompt}'. Use get_page_snapshot(), find the matching element, then use find_element_by_ref() to get its locator. You MUST end by calling report_result() with result_type='locator', success=True, and the locator expression in the 'code' field. Do not provide any additional response after calling report_result()."
         result = await agent.execute_interactive_step(command, {}, max_turns=8)
 
-        if not result.get("success"):
-            raise RuntimeError(f"Could not find the element for {self.prompt}")
+        # First, try to find locator from conversation history (tool results)
+        conversation_history = result.get("conversation_history", [])
+        for turn in conversation_history:
+            # Check for report_result tool calls
+            if turn.tool_calls:
+                for tool_call in turn.tool_calls:
+                    if hasattr(tool_call, "name") and tool_call.name == "report_result":
+                        # Extract the code from report_result arguments
+                        if (
+                            hasattr(tool_call, "input")
+                            and tool_call.input.get("result_type") == "locator"
+                        ):
+                            code = tool_call.input.get("code", "")
+                            if code and code.startswith("page."):
+                                try:
+                                    # Fix naming convention differences between browser API and Python API
+                                    locator_method = code.replace("page.", "self.page.")
+                                    locator_method = locator_method.replace(
+                                        "contentFrame()", "content_frame"
+                                    )
+                                    locator = eval(locator_method)
 
+                                    # Check if the result is a Locator
+                                    if hasattr(locator, "count") and hasattr(
+                                        locator, "click"
+                                    ):
+                                        # Log this for the coding agent to consume
+                                        fallback_info = (
+                                            f" (original failed locator: {self.fallback_locator})"
+                                            if self.fallback_locator
+                                            else ""
+                                        )
+                                        print(
+                                            f"get_by_prompt invoked with '{self.prompt}' generates the following locator: '{code}'{fallback_info}"
+                                        )
+                                        return locator
+                                except Exception:
+                                    continue
+
+        # Fallback to script-based extraction if available
         script = result.get("script", "")
-        if not script:
-            raise RuntimeError(f"No script is found for {self.prompt}")
+        if script:
+            # Find any page expressions that start with page. (including method chaining)
+            matches = re.findall(r"page\.[a-zA-Z_][a-zA-Z0-9_().'\"]*", script)
 
-        # Find any page.method() and check if it returns a Locator
-        matches = re.findall(r"page\.[a-zA-Z_][a-zA-Z0-9_]*\([^)]*\)", script)
-
-        for match in matches:
-            try:
-                locator_method = match.replace("page.", "self.page.")
-                result = eval(locator_method)
-
-                # Check if the result is a Locator
-                if hasattr(result, "count") and hasattr(result, "click"):
-                    # Log this for the coding agent to consume
-                    fallback_info = (
-                        f" (original failed locator: {self.fallback_locator})"
-                        if self.fallback_locator
-                        else ""
+            for match in matches:
+                try:
+                    # Fix naming convention differences between browser API and Python API
+                    locator_method = match.replace("page.", "self.page.")
+                    locator_method = locator_method.replace(
+                        "contentFrame()", "content_frame"
                     )
-                    print(
-                        f"get_by_prompt invoked with '{self.prompt}' generates the following locator: '{match}'{fallback_info}"
-                    )
-                    return result
-            except Exception:
-                continue
+                    locator = eval(locator_method)
 
-        raise Exception(f"Could not locate element with prompt: {self.prompt}")
+                    # Check if the result is a Locator
+                    if hasattr(locator, "count") and hasattr(locator, "click"):
+                        # Log this for the coding agent to consume
+                        fallback_info = (
+                            f" (original failed locator: {self.fallback_locator})"
+                            if self.fallback_locator
+                            else ""
+                        )
+                        print(
+                            f"get_by_prompt invoked with '{self.prompt}' generates the following locator: '{match}'{fallback_info}"
+                        )
+                        return locator
+                except Exception:
+                    continue
+
+        # If we couldn't find a locator but the agent reported success, provide better error
+        if result.get("success"):
+            raise RuntimeError(
+                f"Agent found the element for '{self.prompt}' but couldn't extract a valid locator"
+            )
+        else:
+            raise RuntimeError(f"Could not find the element for {self.prompt}")
 
     def _get_deferred_attribute(self, name):
         """Get an attribute that might require async prompt locator resolution"""
@@ -136,9 +182,19 @@ class PromptBasedLocator:
                     else:
                         return fallback_method(*args, **kwargs)
                 except Exception:
-                    # If fallback fails, continue to prompt-based locator
+                    # Check if VA_DISABLE_FALLBACK is set
+                    if VA_DISABLE_FALLBACK:
+                        # Re-raise the exception instead of falling back to prompt-based locator
+                        raise
+                    # If fallback fails and flag not set, continue to prompt-based locator
                     pass
 
+            # Check if VA_DISABLE_FALLBACK is set before using prompt-based locator
+            if VA_DISABLE_FALLBACK:
+                raise RuntimeError(
+                    "get_by_prompt fallback is disabled by VA_DISABLE_FALLBACK environment variable"
+                )
+
             # Try prompt-based locator
             prompt_locator = await self._get_prompt_locator()
             prompt_method = getattr(prompt_locator, method_name)
