@@ -1,10 +1,11 @@
 import asyncio
 import contextvars
-from typing import Any, Dict, Optional, Callable, Union, Awaitable
+from typing import Any, Dict, Optional, Callable, Union, Awaitable, Type
+from os import environ
 from dataclasses import dataclass
 import logging
 from pathlib import Path
-
+from pydantic import BaseModel
 from playwright._impl._element_handle import ElementHandle as ElementHandleImpl
 from playwright._impl._locator import Locator as LocatorImpl
 from playwright._impl._page import Page as PageImpl
@@ -17,8 +18,8 @@ from playwright.async_api._generated import (
 from playwright.async_api._generated import (
     Page as PageAPI,
 )
+from playwright.async_api import Page as PlaywrightPage
 from playwright.sync_api import Route
-from stagehand import StagehandPage
 
 from .checkpoint import default_checkpoint_callback
 from .locator import PromptBasedLocator
@@ -26,7 +27,7 @@ from .step import AsyncStepContextManager
 from .web_agent import WebAgent
 
 from ..agent.agent import Agent
-from ..constants import REVIEW_TIMEOUT
+from ..constants import REVIEW_TIMEOUT, VA_DISABLE_LOGIN_REVIEW
 from ..review import ReviewStatus, review
 from .dom_utils import (
     inspect_element_recursive,
@@ -50,6 +51,16 @@ class ActResult:
     message: str
 
 
+@dataclass
+class ExtractResult:
+    """Result of a Page.extract operation."""
+
+    # extraction can return a pydantic model or a string
+    extraction: str | BaseModel | None
+    success: bool
+    message: str
+
+
 # Monkey patch the Playwright Page implementation to add snapshotForAI method
 async def _snapshot_for_ai(self, metadata: Optional[Dict[str, Any]] = None) -> str:
     """
@@ -151,8 +162,8 @@ LocatorAPI.generate_locator_string = _api_generate_locator_string  # type: ignor
 
 
 class Page:
-    def __init__(self, page: StagehandPage):
-        self._stagehand_page = page
+    def __init__(self, page: PlaywrightPage):
+        self._playwright_page = page
         self._login_handler = None
         self._agent = Agent()
         # Track any background login tasks (only for fallback case)
@@ -162,6 +173,23 @@ class Page:
         self._warc_route_handler = None
         # Checkpoint review configuration
         self._checkpoint_review_callback = None
+        self._web_agent = WebAgent(self)  # type: ignore
+
+        # Set longer navigation timeout (Playwright default is 3s)
+        self._playwright_page.set_default_navigation_timeout(5000)  # Custom 5s timeout
+
+        # Register default login handler upon page creation
+        if not VA_DISABLE_LOGIN_REVIEW:
+            self.on("login_required")
+
+        # enable checkpoint review in test mode for all pages by default
+        # See https://github.com/orby-ai-engineering/vibe-automation-server/pull/313/files
+        is_test_run = "VA_TEST_RUN" in environ
+        if is_test_run:
+            log.info("Enabling checkpoint review under test run")
+            self.enable_checkpoint_review()
+        else:
+            log.info("Checkpoint review is not enabled for the page")
 
     def get_by_prompt(
         self,
@@ -188,35 +216,45 @@ class Page:
     async def _check_login_and_handle(self, login_handler):
         """Check if login is required and handle it"""
         try:
+            # Set context variable to prevent circular dependency during extract
+            _in_login_handler.set(True)
             # Use extract() to get a text response
-            result = await self._stagehand_page.extract(
-                "Is login required on this page? Answer with exactly 'yes' or 'no' (lowercase, no additional text)."
+            result = await self.extract(
+                "Check if this page requires login by looking for these indicators: "
+                "1) Login/signin forms with username/password fields, "
+                "2) 'Login required' or 'Authentication required' messages, "
+                "3) 'Sign in' or 'Log in' buttons/links, "
+                "4) Messages about accessing 'secure area' or 'protected content', "
+                "5) Login prompts or authentication dialogs. "
+                "Answer with exactly 'yes' if ANY of these login indicators are present, 'no' if none are found."
             )
-            answer = result.extraction.lower()  # type: ignore
+            if result.extraction is None:
+                log.info(
+                    "Extract returned None for login check, assuming no login required"
+                )
+                answer = "no"
+            else:
+                answer = result.extraction.lower()
             if answer == "yes":
                 log.info("Login required detected, calling handler")
                 if login_handler:
                     log.info("Calling login handler")
-                    # Set context variable to mark we're inside a login handler
-                    _in_login_handler.set(True)
-                    try:
-                        # Always pass the page to the handler
-                        if asyncio.iscoroutinefunction(login_handler):
-                            await login_handler(self)
-                        else:
-                            login_handler(self)
-                        log.info("Login handler completed")
-                    finally:
-                        # Reset context variable when done
-                        _in_login_handler.set(False)
+                    # Always pass the page to the handler
+                    if asyncio.iscoroutinefunction(login_handler):
+                        await login_handler(self)
+                    else:
+                        login_handler(self)
+                    log.info("Login handler completed")
                 else:
                     log.warning("Login required but no handler available")
             else:
                 log.info("No login required")
-
         except Exception as e:
             log.error(f"Error in login check: {e}")
             raise Exception(f"Login check failed: {e}") from e
+        finally:
+            # Reset context variable when done
+            _in_login_handler.set(False)
 
     def enable_checkpoint_review(self, callback=None):
         """
@@ -287,8 +325,8 @@ class Page:
             log.info("Background login task completed")
 
     def __getattr__(self, name):
-        """Forward attribute lookups to the underlying Stagehand page."""
-        attr = getattr(self._stagehand_page, name)
+        """Forward attribute lookups to the underlying Playwright page."""
+        attr = getattr(self._playwright_page, name)
 
         # Only wrap callable attributes
         if not callable(attr):
@@ -363,7 +401,7 @@ class Page:
 
             # Only add page load listener if we don't have a handler yet
             if self._login_handler is None:
-                self._stagehand_page.on(
+                self._playwright_page.on(
                     "load", self._check_login_and_handle_on_page_load
                 )
             # if handler is not the same as the existing handler, replace it
@@ -374,7 +412,7 @@ class Page:
             # For other events, handler is required
             if handler is None:
                 raise ValueError(f"Handler is required for event '{event}'")
-            self._stagehand_page.on(event, handler)
+            self._playwright_page.on(event, handler)
 
     def remove_listener(
         self,
@@ -393,13 +431,13 @@ class Page:
                    Can be sync or async and will receive the page as a parameter.
         """
         if event == "login_required":
-            self._stagehand_page.remove_listener(
+            self._playwright_page.remove_listener(
                 "load", self._check_login_and_handle_on_page_load
             )
             self._login_handler = None
             log.info("Login handler removed")
         else:
-            self._stagehand_page.remove_listener(event, handler)
+            self._playwright_page.remove_listener(event, handler)
 
     async def default_login_handler(self, page):
         """Default login handler that starts HITL review"""
@@ -425,13 +463,13 @@ class Page:
         """
         try:
             # Create a web agent for this page
-            web_agent = WebAgent(self._stagehand_page)
+            web_agent = WebAgent(self)
 
             # Use provided context or empty dict
             if context is None:
                 context = {}
 
-            # Execute the action using the web agent
+            # Execute the action using the web agent (use the newly created one!)
             result = await web_agent.execute_interactive_step(prompt, context)
 
             # Return ActResult based on the web agent's result
@@ -443,6 +481,35 @@ class Page:
                 success=False, message=f"Error executing command: {str(e)}"
             )
 
+    async def extract(
+        self,
+        prompt: str,
+        schema: Optional[Type[BaseModel]] = None,
+        include_screenshot: bool = False,
+    ) -> ExtractResult:
+        """
+        Extract data from the page using natural language. If a schema is provided, the extraction will be returned as a model. Otherwise, the extraction will be returned as a string.
+
+        Parameters:
+        -----------
+        prompt (str): Natural language description of what to extract
+        schema (Optional[Type[BaseModel]]): Schema for structured extraction
+
+        Returns:
+        --------
+        ExtractResult: Result with extraction, success status, and message
+        """
+        try:
+            result = await self._web_agent.extract(prompt, schema, include_screenshot)
+
+            # Return ExtractResult based on the web agent's result
+            return ExtractResult(extraction=result, success=True, message="")
+        except Exception as e:
+            log.error(f"Error in extract: {e}")
+            return ExtractResult(
+                extraction=None, success=False, message=f"Error in extract: {e}"
+            )
+
     async def inspect_element(self, x: int, y: int, num_ancestors: int = 3) -> str:
         """
         Inspect an element by coordinates.
@@ -466,7 +533,7 @@ class Page:
             A condensed path representation of the element and its container
         """
         return await inspect_element_recursive(
-            self._stagehand_page, x, y, num_ancestors, mode="element"
+            self._playwright_page, x, y, num_ancestors, mode="element"
         )
 
     async def inspect_html(
@@ -495,7 +562,7 @@ class Page:
             Raw HTML content of the element and its container, formatted like Chrome DevTools
         """
         return await inspect_element_recursive(
-            self._stagehand_page,
+            self._playwright_page,
             x,
             y,
             num_ancestors,
@@ -532,11 +599,8 @@ class Page:
                 # Allow live request
                 await route.continue_()
             else:
-                # Return 404 for URLs not in WARC
-                await route.fulfill(
-                    status=404,
-                    body=f"URL not found in WARC: {request_url}",
-                )
+                # Abort request for URLs not in WARC (follows Playwright default behavior)
+                await route.abort("failed")
 
         # Store handler reference for cleanup
         self._warc_route_handler = warc_route_handler
@@ -552,4 +616,4 @@ class Page:
             self._warc_responses = None
 
     async def get_page_snapshot(self):
-        return self.snapshot_for_ai()
+        return await self.snapshot_for_ai()
