@@ -1,13 +1,13 @@
 import json
 import logging
 from typing import Any, Dict, Optional
-
-from stagehand import StagehandPage
+from playwright.async_api import Page
 
 from va.codegen import (
     inspect_with_block_from_frame,
     record_step_execution,
 )
+from va.constants import VA_DISABLE_FALLBACK
 from ..agent.agent import Agent, create_user_message
 from .web_agent import WebAgent
 from .checkpoint import create_action_checkpoint, _extract_with_block_code
@@ -84,7 +84,7 @@ async def call_llm_for_code_generation(agent: Agent, prompt: str) -> str:
 
 
 async def execute_script(
-    script: str, context_vars: Dict[str, Any], page: StagehandPage
+    script: str, context_vars: Dict[str, Any], page: Page
 ) -> Dict[str, Any]:
     """Execute the generated Python script with the page context."""
     try:
@@ -137,7 +137,7 @@ async def execute_step(
     command: str,
     context: Optional[Dict[str, Any]],
     max_retries: int,
-    page: StagehandPage,
+    page: Page,
     agent: Agent,
 ) -> Dict[str, Any]:
     """
@@ -151,7 +151,7 @@ async def execute_step(
     command (str): Natural language description of the action to perform
     context (Dict[str, Any], optional): Context variables available to the generated script
     max_retries (int): Maximum number of retry attempts (now used as max_turns)
-    page (StagehandPage): The page to execute the command on
+    page (Any): The page to execute the command on
     agent (Agent): The agent to use for LLM calls
 
     Returns:
@@ -198,6 +198,15 @@ class AsyncStepContextManager:
         is_empty_block = self._is_with_block_empty()
 
         if is_empty_block:
+            # Check if VA_DISABLE_FALLBACK is set
+            if VA_DISABLE_FALLBACK:
+                log.info(
+                    f"Empty with block detected but VA_DISABLE_FALLBACK is set, not generating code for: {self.command}"
+                )
+                raise RuntimeError(
+                    f"page.step fallback is disabled by VA_DISABLE_FALLBACK environment variable for command: {self.command}"
+                )
+
             log.info(
                 f"Empty with block detected, triggering LLM action generation for: {self.command}"
             )
@@ -230,12 +239,18 @@ class AsyncStepContextManager:
                 log.error(
                     f"Exception in predefined logic for step '{self.command}': {exc_val}"
                 )
+                # Check if VA_DISABLE_FALLBACK is set
+                if VA_DISABLE_FALLBACK:
+                    log.info(
+                        "VA_DISABLE_FALLBACK is set, not attempting to fix the exception"
+                    )
+                    # Return False to let the exception propagate
+                    return False
                 # Trigger web_agent to handle the exception and propose fixes
                 await self._handle_predefined_logic_exception(exc_type, exc_val, exc_tb)
                 # Return True to suppress the exception since we handled it
                 return True
             else:
-                log.info(f"Manual execution detected for step: {self.command}")
                 self.step_result = {
                     "success": True,
                     "message": "Manual execution completed",
@@ -376,4 +391,4 @@ The fixed code should accomplish the original command while handling the error c
 
     def __getattr__(self, name):
         """Forward attribute access to delegate to page."""
-        return getattr(self.page._stagehand_page, name)
+        return getattr(self.page._playwright_page, name)
