@@ -2,6 +2,7 @@ from typing import Any
 from abc import ABC, abstractmethod
 
 from va.models import ReviewModal
+from va.protos.orby.va.public.execution_messages_pb2 import ExecutionStatus
 
 
 class ExecutionStore(ABC):
@@ -10,7 +11,17 @@ class ExecutionStore(ABC):
         pass
 
     @abstractmethod
-    def mark_stop(self, execution_id: str) -> None:
+    def mark_stop(
+        self, execution_id: str, status: ExecutionStatus.ValueType = None
+    ) -> None:
+        pass
+
+    @abstractmethod
+    def mark_for_review(self, execution_id: str) -> None:
+        pass
+
+    @abstractmethod
+    def mark_resume(self, execution_id: str) -> None:
         pass
 
     @abstractmethod
