@@ -10,7 +10,7 @@ from pydantic import BaseModel, ValidationError
 from va.automation import Automation
 from va.store import get_store
 from va.logging_handlers import ExecutionLogHandler
-from va.utils import is_test_execution
+from va.utils import is_test_execution, TEST_EXECUTION_PREFIX
 from va.protos.orby.va.public.execution_messages_pb2 import ExecutionStatus
 
 
@@ -122,17 +122,11 @@ def _is_pydantic_model(type_hint):
 def _create_managed_logger(execution_id: str):
     """Create (or fetch) a managed logger instance for the execution.
 
-    The logger is equipped with two handlers:
-    1. A StreamHandler so that logs still appear on stdout/stderr
-    2. An ExecutionLogHandler that forwards the message to
-    the backend when we have a managed, non-test execution
+    The logger is equipped with a A StreamHandler so that logs still appear on stdout/stderr
     """
 
     logger = logging.getLogger(f"va.execution.{execution_id}")
 
-    if not is_test_execution(execution_id):
-        logger.addHandler(ExecutionLogHandler(execution_id))
-
     if not any(isinstance(h, logging.StreamHandler) for h in logger.handlers):
         stream_handler = logging.StreamHandler()
         stream_handler.setFormatter(
@@ -197,10 +191,17 @@ def workflow(workflow_name: str):
                 execution_id = os.environ["VA_EXECUTION_ID"]
                 is_managed_execution = True
             else:
-                execution_id = str(uuid.uuid4())
+                execution_id = TEST_EXECUTION_PREFIX + "-" + str(uuid.uuid4())
                 is_managed_execution = False
 
             store = get_store(is_managed_execution)
+
+            # Attach a single root-level ExecutionLogHandler bound to this execution
+            # (if not a local test execution) so all relevant logs are forwarded to the backend
+            if not is_test_execution(execution_id):
+                root_logger = logging.getLogger()
+                root_logger.addHandler(ExecutionLogHandler(execution_id))
+
             automation = Automation(store, workflow_name, execution_id)
             Automation.set_instance(automation)
             automation.execution.mark_start()
